import React, { useEffect, useRef, useState } from "react";
import PageTitle from "../../components/PageTitle";
import Container from "../../components/Container";
import Sidebar from "../../components/sidebar";
import Filter from "../../components/sidebar/Filter";
import SelectInput from "../../components/form/SelectInput";
import NoDataFound from "../../components/common/NoDataFound";
import SingleProduct from "../../components/ProductDetails/SingleProduct";
import { useShopPageInfoQuery } from "../../app/features/products/apiSlice";
import Head from "next/head";
import LoadingCom from "../../components/common/loadingCom";
import useLanguage from "../../hooks/useLanguage";

function Listing({ info, category, search }) {
  const [page, setPage] = useState(1);
  const [filter, setFilter] = useState(false);
  const [priceRange, setPriceRange] = useState([10]);
  const [workingPrice, setWorkingPrice] = useState(priceRange);

  //service filters
  const [serviceFilters, setServiceFilters] = useState(info?.service);

  const handleServiceFilters = (e) => {
    setServiceFilters((prev) =>
      prev.map((item) =>
        item.id == e.target.name
          ? { ...item, active: !item.active }
          : { ...item }
      )
    );
    setPage(1);
  };

  // suppliers filters
  const [suppliersFilters, setSuppliersFilters] = useState(info?.suppliers);
  const handleSuppliersFilters = (e) => {
    setSuppliersFilters((prev) =>
      prev.map((item) =>
        item.id == e.target.name
          ? { ...item, active: !item.active }
          : { ...item }
      )
    );
    setPage(1);
  };

  // Specifications filters

  const [specFilters, setSpecFilters] = useState({});
  const handleSpecFilters = (info) => {
    setSpecFilters((prev) => ({ ...prev, ...info }));
  };
  // clear spec filters

  const handleSpecClear = () => setSpecFilters({});

  // clearFilter
  const handleClearFilter = () => {
    setPriceRange([10]);
    setServiceFilters(info?.service);
    setSuppliersFilters(info?.suppliers);
  };

  // Range handler

  const rangeHandler = (e) => {
    let temp = [...priceRange];
    temp[e.target.name] = e.target.value;
    setPriceRange([...temp]);
  };
  rangeHandler.bind(this);

  const priceRangeHandler = (value) => {
    setPriceRange(value);
  };

  useEffect(() => {
    const timeoutId = setTimeout(() => {
      setWorkingPrice(priceRange);
      setPage(1);
    }, 1000);
    return () => clearTimeout(timeoutId);
  }, [priceRange]);

  const { data, isFetching } = useShopPageInfoQuery({
    page: page,
    suppliersFilters: suppliersFilters,
    priceRange: workingPrice,
    serviceFilters: serviceFilters,
    specFilters: specFilters,
    category: category,
    search: search,
  });
  const observerTarget = useRef(null);
  useEffect(() => {
    const observer = new IntersectionObserver(
      (entries) => {
        if (entries[0].isIntersecting) {
          if (!isFetching && data?.orders?.total > data?.orders?.to) {
            setPage((prev) => prev + 1);
          }
        }
      },
      { threshold: 1 }
    );

    if (observerTarget.current) {
      observer.observe(observerTarget.current);
    }

    return () => {
      if (observerTarget.current) {
        observer.unobserve(observerTarget.current);
      }
    };
  }, [observerTarget, isFetching, data?.orders]);
  const language = useLanguage();

  return (
    <>
      <Head>
        <title>{info?.seoSetting?.seo_title}</title>
      </Head>
      <div className="bg-[#FDF9FF] w-full relative pb-20">
        <PageTitle
          title={info?.seoSetting?.page_name}
          pageName={info?.seoSetting?.seo_title}
        />
        <Container>
          <div className="flex pt-[60px] lg:gap-10 gap-0">
            <div className=" absolute lg:static z-40">
              <Sidebar isOpen={filter}>
                <Filter
                  clearFilters={handleClearFilter}
                  close={() => setFilter(false)}
                  suppliers={suppliersFilters}
                  services={serviceFilters}
                  handleServiceFilters={handleServiceFilters}
                  handleSuppliersFilters={handleSuppliersFilters}
                  priceRange={priceRange}
                  rangeHandler={rangeHandler}
                  setPriceRange={priceRangeHandler}
                />
              </Sidebar>
            </div>
            <div className="w-full">
              {/* listing filter start */}
              <div
                className="bg-white xl:p-[30px] p-3 hidden sm:block"
                data-aos="fade-left"
              >
                <div className="flex w-full justify-between mb-[14px]">
                  <h2 className=" text-[22px] text-dark-300 font-bold notranslate">
                    {language["Specifications"]}
                  </h2>
                  <button
                    onClick={handleSpecClear}
                    className=" text-primary-200 text-sm font-medium self-end notranslate"
                  >
                    <p>{language["Clear all filters"]}</p>
                  </button>
                </div>
                <div className="flex w-full flex-wrap gap-5">
                  {info?.specifications?.map((spec, index) => (
                    <SelectInput
                      key={spec.key + index}
                      border="#F8EBFF"
                      name={spec.key}
                      value={specFilters?.[spec.key]?.name || spec?.key}
                      handler={handleSpecFilters}
                      padding="p-2"
                      options={spec?.specifications?.map((item) => ({
                        name: item.specification,
                        slug: item.product_specification_key_id,
                      }))}
                    />
                  ))}
                </div>
              </div>
              {/* listing filter end */}
              {data?.products?.data?.length > 0 ? (
                <div className="flex flex-col w-full xl:mt-11 sm:mt-5 gap-5">
                  <div className="flex gap-4 flex-col-reverse xl:flex-row w-full justify-between">
                    <p className=" text-dark-300 text-base font-medium notranslate">
                      {language["Showing"]}{" "}
                      <span className=" text-primary-200">
                        {" "}
                        {data?.products?.to
                          ? data?.products?.to
                          : info?.products?.to}
                      </span>{" "}
                      of{" "}
                      <span className=" text-primary-200">
                        {" "}
                        {info?.products?.total}{" "}
                      </span>
                      {language["results"]}
                    </p>
                    <div className="flex flex-col-reverse gap-2 justify-between sm:flex-row-reverse ">
                      <div className="flex gap-4 "></div>
                      <button
                        className="secondaryBtn w-fit xl:hidden notranslate"
                        onClick={() => setFilter((prev) => !prev)}
                      >
                        {language["Filter"]}{" "}
                        <svg
                          xmlns="http://www.w3.org/2000/svg"
                          height="16"
                          width="18"
                          viewBox="0 0 576 512"
                        >
                          <path d="M151.6 469.6C145.5 476.2 137 480 128 480s-17.5-3.8-23.6-10.4l-88-96c-11.9-13-11.1-33.3 2-45.2s33.3-11.1 45.2 2L96 365.7V64c0-17.7 14.3-32 32-32s32 14.3 32 32V365.7l32.4-35.4c11.9-13 32.2-13.9 45.2-2s13.9 32.2 2 45.2l-88 96zM320 32h32c17.7 0 32 14.3 32 32s-14.3 32-32 32H320c-17.7 0-32-14.3-32-32s14.3-32 32-32zm0 128h96c17.7 0 32 14.3 32 32s-14.3 32-32 32H320c-17.7 0-32-14.3-32-32s14.3-32 32-32zm0 128H480c17.7 0 32 14.3 32 32s-14.3 32-32 32H320c-17.7 0-32-14.3-32-32s14.3-32 32-32zm0 128H544c17.7 0 32 14.3 32 32s-14.3 32-32 32H320c-17.7 0-32-14.3-32-32s14.3-32 32-32z" />
                        </svg>
                      </button>
                    </div>
                  </div>
                  {data?.products?.data?.map((product, index) => (
                    <SingleProduct
                      key={product?.id + index}
                      product={product}
                      activeWarehouse={true}
                    />
                  ))}
                </div>
              ) : !isFetching ? (
                <NoDataFound />
              ) : (
                <LoadingCom />
              )}
              <div ref={observerTarget}></div>
            </div>
          </div>
        </Container>
      </div>
    </>
  );
}

export default Listing;

export async function getServerSideProps({ query }) {
  const res = await fetch(`${process.env.BASE_URL}api/product`);
  const data = await res.json();
  return { props: { info: data, ...query } };
}
